class DontStopPlaying extends Mutator config(DontStopPlaying);

struct PlayerTracker
{
	var string		IP;//track per IP
	var byte 		warn_count;//how many warnings been given in this game?
	var int			last_activity;//Level.TimeSeconds of last change in location detected
	var vector		last_location;//location at that time
	var bool 		been_warned;//warned on current idle?
};

var config int MAX_TIME;//how much time of nonactivity before the player gets kicked?
var config int WARN_TIME;//when (< MAX_TIME) to warn the player
var config int WARN_LIMIT;//instead of giving the WARN_LIMIT. warning -> kick ban for this game

var PlayerTracker 	PT[64];
var byte 			num_PTs;

function PostBeginPlay()
{
	SetTimer(1, True);
}

function ModifyPlayer(Pawn pwn)
{
	local byte index;

	if(NextMutator != None)
		NextMutator.ModifyPlayer(pwn);	

	if(!pwn.isA('TournamentPlayer'))
		return;

	index = getIndex(pwn);
	if(index < 0)
		return;//can't handle more

	if(PT[index].warn_count == WARN_LIMIT)//no comeback
	{
		pwn.ClientMessage("Try playing actually on the next game, or join spectators.");
		pwn.Destroy();
		return;
	}

	//spawn = action
	PT[index].last_activity = Level.TimeSeconds;
	PT[index].last_location = pwn.Location;
	PT[index].been_warned =	False;
}

function Timer()
{
	local byte i, index;
	local TournamentPlayer tp;

	//check all players
	foreach AllActors(Class'TournamentPlayer',tp)
	{
		index = getIndex(tp);
		if(index < 0)
			continue;

		//moved?
		if(PT[index].last_location != tp.Location)	
		{
			PT[index].last_location = tp.Location;
			PT[index].last_activity = Level.TimeSeconds;
			PT[index].been_warned = False;//next try
		}
		else//did not move
		{
			if(Level.TimeSeconds - PT[index].last_activity > WARN_TIME)
			{
				//give warning now?
				if(!PT[index].been_warned)
				{
					PT[index].been_warned = True;
					PT[index].warn_count ++;
					
					if(PT[index].warn_count < WARN_LIMIT)
						tp.ClientMessage("[WARNING] Start playing or get kicked! " $ PT[index].warn_count $ " / " $ WARN_LIMIT, 'CriticalEvent', True);
					else//kick on reaching WARN_LIMIT
					{
						tp.ClientMessage("[KICK] Enough warnings. Wait for next game and try playing then.");
						tp.Destroy();
					}
					continue;
				}
				//repeat
				tp.ClientMessage("[WARNING] Start playing or get kicked! " $ PT[index].warn_count $ " / " $ WARN_LIMIT, 'CriticalEvent', True);
			}
			
			//kick on max idletime
			if(Level.TimeSeconds - PT[index].last_activity > MAX_TIME)
			{
				tp.ClientMessage("[KICK] You didn't play for too long. Reconnect if you are ready to do so.");
				tp.Destroy();
			}
		}
	}
}

function byte getIndex(Pawn pwn)
{
	local byte 		i;
	local string	pawnIP;

	pawnIP = PlayerPawn(pwn).GetPlayerNetworkAddress();
	pawnIP = Left(pawnIP, InStr(pawnIP, ":"));

	//already there in the list?
	for(i = 0;i < num_PTs;i++)
	{
		if(pawnIP == PT[i].IP)
			return i;
	}
	//not on the list; try new entry
	if(num_PTs < 64)
	{
		PT[num_PTs].IP = pawnIP;
		PT[num_PTs].warn_count = 0;
		num_PTs ++;
		return num_PTs - 1;
	}
	return -1;//can't take player on the list
}

defaultproperties
{
	MAX_TIME=40
	WARN_TIME=32
	WARN_LIMIT=3
}
